#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Permissions API Endpoint"""

from acapi2.exceptions import AcquiaCloudNoDataException
from acapi2.resources.acquialist import AcquiaList


class PermissionsList(AcquiaList):
    def __init__(
        self, uri: str, api_key: str, api_secret: str, *args, **kwargs
    ) -> None:
        super().__init__(uri, api_key, api_secret, *args, **kwargs)
        self.fetch()

    def fetch(self) -> None:
        perms = self.request(uri=self.uri).json()

        try:
            perm_items = perms["_embedded"]["items"]
        except KeyError:
            raise AcquiaCloudNoDataException
        else:
            for perm in perm_items:
                name = perm["name"]
                self.__setitem__(name, perm)

    @property
    def base_uri(self) -> str:
        return self._base_uri

    @base_uri.setter
    def base_uri(self, base_uri: str):
        self._base_uri = f"{base_uri}/permissions"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Manipulate environments and perform related actions."""

from requests.sessions import Session

from acapi2.resources.acquiaresource import AcquiaResource


class Environment(AcquiaResource):
    def backups(self, db_name: str) -> Session:
        """
        Return a list of backups.

        :param db_name: Database name, typically lower snake case.
        """
        uri = f"{self.uri}/databases/{db_name}/backups"
        return self.request(uri=uri, method="GET").json()

    def backup_details(self, db_name: str, backup_id: str) -> Session:
        """
        Return details about a specific backup
        :param db_name: Database name, typically lower snake case.
        :param backup_id: Database backup id.
        """
        uri = f"{self.uri}/databases/{db_name}/backups/{backup_id}"
        return self.request(uri=uri, method="GET").json()

    def backup_download(self, db_name: str, backup_id: str) -> Session:
        """
        Get link to be able to download backup.

        :param db_name: Database name, typically lower snake case.
        :param backup_id: Database backup id.
        :return: Acquia response
        """
        uri = (
            f"{self.uri}/databases/{db_name}/backups/{backup_id}/"
            f"actions/download"
        )
        return self.request(uri=uri, method="GET").json()

    def code_switch(self, branch_tag: str) -> Session:
        """
        Switch code on this environment to a different branch or release tag.

        :param branch_tag: The tag to switch to.
        """
        uri = f"{self.uri}/code/actions/switch"
        data = {"branch": branch_tag}

        response = self.request(uri=uri, method="POST", data=data)
        return response

    def configure(self, data: dict) -> Session:
        """
        Modify configuration settings for an environment.

        :param data: Configuration parameters.
        """
        return self.request(uri=self.uri, method="PUT", data=data)

    def create_backup(self, db_name: str) -> Session:
        """
        Create a backup

        :param db_name: Database name, typically lower snake case.
        """
        uri = f"{self.uri}/databases/{db_name}/backups"
        return self.request(uri=uri, method="POST", data={}).json()

    def create_domain(self, domain: str) -> Session:
        """
        Add a domain to the environment.

        :param domain: Domain to add to the environment.
        """
        uri = f"{self.uri}/domains"
        data = {"hostname": domain}
        response = self.request(uri=uri, method="POST", data=data)

        return response

    def create_log_forwarding_destinations(
        self,
        label: str,
        sources: list,
        consumer: str,
        credentials: dict,
        address: str,
    ) -> Session:
        """
        Create a log forwarding destination.
        :param label: Human-friendly identifier of the destination
        :param sources: List of log sources to forward
        :param consumer: Application or provider consuming the logs.
        :param credentials: Dictionary of certificate,key and token.
        :param address: URL or host name and port of the destination.
        :param destination_uuid: Log forwarding destination uuid for given env.
        """
        uri = f"{self.uri}/log-forwarding-destinations"
        data = {
            "label": label,
            "sources": sources,
            "consumer": consumer,
            "credentials": credentials,
            "address": address,
        }
        response = self.request(uri=uri, method="POST", data=data)

        return response

    def databases(self) -> Session:
        """
        Retrieve a list of databases.
        """
        uri = f"{self.uri}/databases"
        return self.request(uri=uri, method="GET").json()

    def delete_backup(self, db_name: str, backup_id: str) -> Session:
        """
        Delete a backup

        :param db_name: Database name, typically lower snake case.
        :param backup_id: Database backup id.
        """
        uri = f"{self.uri}/databases/{db_name}/backups/{backup_id}"
        return self.request(uri=uri, method="DELETE").json()

    def delete_domain(self, domain: str) -> Session:
        """
        Remove the domain from the environment.

        :param domain: Domain name to delete.
        """
        uri = f"{self.uri}/domains/{domain}"
        response = self.request(uri=uri, method="DELETE")

        return response

    def delete_log_forwarding_destinations(
        self,
        destination_uuid: str,
    ) -> Session:
        """
        Delete a log forwarding destination.

        :param destination_uuid: log forwarding destination uuid for given env.
        """
        uri = f"{self.uri}/log-forwarding-destinations/{destination_uuid}"
        response = self.request(uri=uri, method="DELETE")

        return response

    def clear_varnish_domain(self, domain: str) -> Session:
        """
        Clear the Varnish cache for the domain attached to this environment.

        :param domain: Domain name.
        """
        uri = f"{self.uri}/domains/{domain}/actions/clear-varnish"
        data = {"hostname": domain}
        response = self.request(uri=uri, method="POST", data=data)

        return response

    def clear_varnish_domains(self, domains: list) -> Session:
        """
        Clear the Varnish cache for multiple domains
         attached to this environment.

        :param domains: Domain name list.
        """
        uri = f"{self.uri}/domains/actions/clear-varnish"
        data = {"domains": domains}

        response = self.request(uri=uri, method="POST", data=data)

        return response

    def destroy(self):
        """
        Delete a CD environment.
        """
        response = self.request(uri=self.uri, method="DELETE")

        return response

    def deploy_code(self, id_from: str) -> Session:
        """
        Deploy code to this environment.

        :param id_from: uuid for the environment to deploy code from.
        """
        uri = f"{self.uri}/code"
        data = {"source": id_from}

        response = self.request(uri=uri, method="POST", data=data)
        return response

    def deploy_database(self, id_from: str, db_name: str) -> Session:
        """
        Copy a database to this environment.

        :param id_from: uuid for the environment to deploy the db from.
        :param db_name: the database name to use.
        """
        uri = f"{self.uri}/databases"
        data = {"name": db_name, "source": id_from}

        response = self.request(uri=uri, method="POST", data=data)
        return response

    def deploy_files(self, id_from: str) -> Session:
        """
        Copy files to this environment.

        :param id_from: uuid for the environment to deploy the files from.
        """
        uri = f"{self.uri}/files"
        data = {"source": id_from}

        response = self.request(uri=uri, method="POST", data=data)
        return response

    def get_crons(self) -> dict:
        """
        Return a list of the cron jobs on an environment.
        """
        uri = f"{self.uri}/crons"

        response = self.request(uri=uri)
        return response.json()

    def get_log_forwarding_destinations(self) -> dict:
        """
        Return a collection of log forwarding destinations.
        """
        uri = f"{self.uri}/log-forwarding-destinations"

        response = self.request(uri=uri)
        return response.json()

    def get_servers(self) -> dict:
        """
        Return a list of servers.
        """
        uri = f"{self.uri}/servers"

        response = self.request(uri=uri)
        return response.json()

    def get_php_version(self) -> dict:
        """
        Get the PHP version number.
        """
        uri = f"{self.uri}/"

        response = self.request(uri=uri)
        env_config = response.json()
        return {"php_version": env_config["configuration"]["php"]["version"]}

    def set_php_version(self, version: str) -> Session:
        """
        Set the PHP version.

        :param version: PHP version number to use.
        """
        data = {"version": version}

        return self.configure(data)

    def update_log_forwarding_destinations(
        self,
        label: str,
        sources: list,
        consumer: str,
        credentials: dict,
        address: str,
        destination_uuid: str,
    ) -> Session:
        """
        Update a log forwarding destination.
        :param label: Human-friendly identifier of the destination
        :param sources: List of log sources to forward
        :param consumer: Application or provider consuming the logs.
        :param credentials: Dictionary of certificate,key and token.
        :param address: URL or host name and port of the destination.
        :param destination_uuid: Log forwarding destination uuid for given env.
        """
        uri = f"{self.uri}/log-forwarding-destinations/{destination_uuid}"
        data = {
            "label": label,
            "sources": sources,
            "consumer": consumer,
            "credentials": credentials,
            "address": address,
        }
        response = self.request(uri=uri, method="PUT", data=data)

        return response

    def delete_cron(self, cron_id: str) -> Session:
        """
        Deletes a cron job.
        :cron_id: An ID that uniquely identifies a cron job.
        """
        uri = f"{self.uri}/crons/{cron_id}"
        response = self.request(uri=uri, method="DELETE")

        return response

    def disable_cron(self, cron_id: str) -> Session:
        """
        Disables a cron job.
        :cron_id: An ID that uniquely identifies a cron job.
        """
        uri = f"{self.uri}/crons/{cron_id}/actions/disable"
        response = self.request(uri=uri, method="POST", data="")

        return response

    def enable_cron(self, cron_id: str) -> Session:
        """
        Enables a cron job.
        :cron_id: An ID that uniquely identifies a cron job.
        """
        uri = f"{self.uri}/crons/{cron_id}/actions/enable"
        response = self.request(uri=uri, method="POST", data="")

        return response

    def get_ssl_settings(self) -> dict:
        """
        Return the SSL settings for the environment.
        """
        uri = f"{self.uri}/ssl"
        response = self.request(uri=uri)

        return response.json()

    def get_ssl_certs(self) -> dict:
        """
        Return a list of SSL certificates.
        """
        uri = f"{self.uri}/ssl/certificates"
        response = self.request(uri=uri)

        return response.json().get("_embedded", {}).get("items")

    def get_ssl_cert(self, cert_id: str) -> dict:
        """
        Return an SSL cert.
        """
        uri = f"{self.uri}/ssl/certificates/{cert_id}"
        response = self.request(uri=uri)

        return response.json()

    def install_ssl_cert(
        self,
        label: str,
        certificate: str,
        private_key: str,
        ca_certificates: str = None,
        legacy: bool = False,
        csr_id: int = None,
    ) -> Session:
        """
        Add a new SSL cert to the environment.
        :param: label: Human-friendly identifier for the cert.
        :param: certificate: The certificate in PEM format.
        :param: private_key: The private keyfor the cert in PEM format.
        :param: ca_certificates: Any chain certificates, in PEM format.
         Defaults to None.
        :param: legacy: Legacy in the sense of Acquia's legacy architecture,
         not an old version of the SSL or TLS standards. See Acquia's docs
         for more details. Defaults to False.
        :param: csr_id: Associate with an existing installed CSR. Defaults
         to None.
        """
        uri = f"{self.uri}/ssl/certificates"
        data = {
            "legacy": legacy,
            "label": label,
            "certificate": certificate,
            "private_key": private_key,
        }
        if csr_id is not None:
            data["csr_id"] = csr_id
        if ca_certificates is not None:
            data["ca_certificates"] = ca_certificates
        response = self.request(uri=uri, method="POST", data=data)

        return response

    def delete_ssl_cert(self, cert_id: str) -> Session:
        """
        Remove an SSL cert.
        :param: cert_id: The Acquia certificate ID.
        """
        uri = f"{self.uri}/ssl/certificates/{cert_id}"
        response = self.request(uri=uri, method="DELETE")

        return response

    def activate_ssl_cert(self, cert_id: str) -> Session:
        """
        Activate a previously installed SSL cert.
        :param: cert_id: The Acquia certificate ID.
        """
        uri = f"{self.uri}/ssl/certificates/{cert_id}/actions/activate"
        response = self.request(uri=uri, method="POST", data={})

        return response

    def deactivate_ssl_cert(self, cert_id: str) -> Session:
        """
        Deactivate a previously installed SSL cert.
        :param: cert_id: The Acquia certificate ID.
        """
        uri = f"{self.uri}/ssl/certificates/{cert_id}/actions/deactivate"
        response = self.request(uri=uri, method="POST", data={})

        return response



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Acquia Cloud API Data network resource"""
import logging
import time
import uuid
from platform import python_version
from pprint import pformat
from urllib.parse import urlencode

import backoff
import httphmac
import requests

from acapi2.http_request import HttpRequest
from acapi2.version import __version__

_logger = logging.getLogger("acapi2.resources.acquiadata")


class AcquiaData(object):
    _agent = "Acquia Cloud API Client/{mver} (Python {pver})"
    _realm = _agent.format(mver=__version__, pver=python_version())

    def __init__(
        self, uri: str, api_key: str, api_secret: str, data: dict = None
    ) -> None:
        self.uri = uri
        self.api_key = api_key
        self.api_secret = api_secret
        self._data = data or {}
        self.last_response: requests.Response = requests.Response()

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, data: dict):
        self._data = data

    @staticmethod
    def generate_url_query(uri: str, params: dict) -> str:
        # TODO: add params support in httphmac.Request()
        qry = urlencode(params)
        url = "{uri}?{qry}".format(uri=uri, qry=qry)
        return url

    @backoff.on_exception(
        backoff.expo, requests.exceptions.RequestException, max_time=10
    )
    def request(
        self,
        uri: str = None,
        method: str = "GET",
        data: dict = None,
        params: dict = None,
    ):

        if not uri:
            uri = self.uri

        if params:
            params = {k: v for k, v in params.items() if v is not None}
            uri = self.generate_url_query(uri, params)

        response: requests.Response
        request = HttpRequest()
        auth_headers = {
            "realm": self._realm,
            "id": self.api_key,
            "nonce": uuid.uuid4().hex,
            "version": "2.0",
        }
        request.with_url(uri).with_method(method).with_time()
        signer = httphmac.V2Signer()
        signer.sign_direct(request, auth_headers, self.api_secret)

        if "GET" == method:
            attempt = 0
            while attempt <= 5:
                response = request.do()
                if response.status_code not in list(range(500, 505)):
                    break

                attempt += 1
                time.sleep((attempt ** 2.0) / 10)

        if "POST" == method or "PUT" == method:
            request.with_json_body(data)
            signer.sign_direct(request, auth_headers, self.api_secret)
            response = request.do()
            logging.debug(response.content)

        if "DELETE" == method:
            response = request.do()
            logging.debug(response.content)

        self.last_response = response
        if (
            response.status_code != requests.codes.ok
            and response.status_code != requests.codes.accepted
        ):
            try:
                response.raise_for_status()
            except requests.exceptions.HTTPError as exp:
                _logger.warning(
                    "Failed request response headers: \n%s",
                    pformat(exp.response.headers, indent=2),
                )
                raise

        return response



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Acquia Cloud API task resources."""

from acapi2.resources.acquialist import AcquiaList
from acapi2.resources.task import Task


class TaskList(AcquiaList):
    def __init__(
        self,
        uri: str,
        api_key: str,
        api_secret: str,
        filters: dict = None,
        *args,
        **kwargs,
    ) -> None:
        super().__init__(uri, api_key, api_secret, *args, **kwargs)
        self._filters = filters
        self.fetch()

    def fetch(self) -> None:
        tasks = self.request(uri=self.uri, params=self._filters).json()
        try:
            task_items = tasks["_embedded"]["items"]
        except KeyError:
            # TODO Handle this
            pass
        else:
            for task in task_items:
                task_id = task["uuid"]
                self.__setitem__(
                    task_id, Task(self.uri, self.api_key, self.api_secret)
                )
                loaded_task = self.__getitem__(task_id)
                loaded_task.data = task

    @property
    def base_uri(self) -> str:
        return self._base_uri

    @base_uri.setter
    def base_uri(self, base_uri: str):
        self._base_uri = f"{base_uri}/tasks"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Applications tests"""

import requests
import requests_mock

from acapi2.resources.applicationlist import ApplicationList
from acapi2.resources.environmentlist import EnvironmentList
from acapi2.tests import BaseTest


@requests_mock.Mocker()
class TestApplications(BaseTest):
    def test_application_list(self, mocker):
        response = {
            "total": 2,
            "pagination": {"total": 2, "limit": 10, "offset": 0},
            "_links": {
                "self": {"href": "{baseUri}/applications?limit=10"},
                "sort": {
                    "href": "{baseUri}/applications{?sort}",
                    "templated": True,
                },
                "filter": {
                    "href": "{baseUri}/applications{?filter}",
                    "templated": True,
                },
                "limit": {
                    "href": "{baseUri}/applications{?limit}",
                    "templated": True,
                },
                "parent": {"href": "{baseUri}"},
            },
            "_embedded": {
                "items": [
                    {
                        "id": 241643,
                        "uuid": "a47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                        "name": "Sample application 1",
                        "hosting": {"type": "acp", "id": "devcloud:devcloud2"},
                        "subscription": {
                            "uuid": "s47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                            "name": "Sample subscription",
                        },
                        "organization": {
                            "uuid": "g47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                            "name": "Sample organization",
                        },
                        "flags": {
                            "remote_admin": True,
                            "service_management": True,
                        },
                        "type": "drupal",
                        "status": "normal",
                        "_links": {
                            "self": {
                                "href": "{baseUri}/applications/"
                                "a47ac10b-58cc-4372"
                                "-a567-0e02b2c3d470"
                            }
                        },
                    },
                    {
                        "id": 954291,
                        "uuid": "a47ac10b-58cc-4372" "-a567-0e02b2c3d471",
                        "name": "Sample application 2",
                        "hosting": {
                            "type": "free",
                            "id": "devcloud:devcloud2",
                        },
                        "subscription": {
                            "uuid": "s47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                            "name": "Sample subscription",
                        },
                        "organization": {
                            "uuid": "g47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                            "name": "Sample organization",
                        },
                        "type": "node",
                        "flags": {
                            "remote_admin": False,
                            "service_management": False,
                        },
                        "status": "provisioning",
                        "_links": {
                            "self": {
                                "href": "{baseUri}/applications/"
                                "a47ac10b-58cc-4372"
                                "-a567-0e02b2c3d471"
                            }
                        },
                    },
                ]
            },
        }

        uri = "{}/applications".format(self.endpoint)
        mocker.register_uri("GET", uri, json=response, status_code=200)

        apps = self.acquia.applications()
        self.assertIsInstance(apps, ApplicationList)

    def test_applications_filters(self, mocker):
        pass

    def test_application_not_found(self, mocker):
        app_uuid = "a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/applications/{uuid}"
        uri = uri.format(base_uri=self.endpoint, uuid=app_uuid)

        mocker.register_uri(url=uri, method="GET", status_code=404)

        acquia_data = self.acquia.application(app_uuid)

        with self.assertRaises(requests.HTTPError):
            acquia_data.request()

    def test_create_environment(self, mocker):
        response = {"message": "Adding an environment."}

        app_uuid = "a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/applications/{uuid}/environments"
        uri = uri.format(base_uri=self.endpoint, uuid=app_uuid)

        mocker.register_uri(
            url=uri, method="POST", status_code=202, json=response
        )

        response = self.acquia.application(app_uuid).create_environment(
            "Ticket Numer 123", "ticket-number-123", "ticket-number-123"
        )

        self.assertEqual(response.status_code, 202)

    def test_get_environments(self, mocker):
        response = {
            "total": 3,
            "_links": {
                "self": {
                    "href": "{baseUri}/applications/"
                    "a47ac10b-58cc-4372"
                    "-a567-0e02b2c3d470/environments"
                },
                "sort": {
                    "href": "{baseUri}/applications/"
                    "a47ac10b-58cc-4372"
                    "-a567-0e02b2c3d470/environments{?sort}",
                    "templated": True,
                },
                "filter": {
                    "href": "{baseUri}/applications/"
                    "a47ac10b-58cc-4372"
                    "-a567-0e02b2c3d470/"
                    "environments{?filter}",
                    "templated": True,
                },
                "limit": {
                    "href": "{baseUri}/applications/"
                    "a47ac10b-58cc-4372"
                    "-a567-0e02b2c3d470/environments{?limit}",
                    "templated": True,
                },
                "parent": {
                    "href": "{baseUri}/applications/"
                    "a47ac10b-58cc-4372-a567-0e02b2c3d470"
                },
            },
            "_embedded": {
                "items": [
                    {
                        "id": "24-a47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                        "label": "Dev",
                        "name": "dev",
                        "application": {
                            "name": "Sample Drupal application",
                            "uuid": "a47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                        },
                        "domains": [
                            "sitedev.hosted.acquia-sites.com",
                            "example.com",
                        ],
                        "active_domain": "example.com",
                        "default_domain": "sitedev.hosted" ".acquia-sites.com",
                        "image_url": None,
                        "ips": ["10.0.1.5"],
                        "region": "us-east-1",
                        "status": "normal",
                        "type": "drupal",
                        "size": None,
                        "weight": 0,
                        "vcs": {
                            "type": "git",
                            "path": "master",
                            "url": "site@svn-3.hosted."
                            "acquia-sites.com:site.git",
                        },
                        "insight": {
                            "status": "ok",
                            "scores": {"insight": 87, "seo": 56},
                            "counts": {
                                "best_practices": {
                                    "pass": 7,
                                    "fail": 0,
                                    "ignored": 0,
                                    "total": 7,
                                    "percent": 100,
                                },
                                "performance": {
                                    "pass": 22,
                                    "fail": 0,
                                    "ignored": 0,
                                    "total": 22,
                                    "percent": 100,
                                },
                                "security": {
                                    "pass": 23,
                                    "fail": 0,
                                    "ignored": 0,
                                    "total": 23,
                                    "percent": 100,
                                },
                            },
                        },
                        "flags": {
                            "cde": False,
                            "hsd": False,
                            "livedev": False,
                            "multi_region": False,
                            "production": False,
                            "production_mode": False,
                            "remote_admin": False,
                            "varnish": True,
                            "varnish_over_ssl": False,
                            "service_management": True,
                        },
                        "configuration": {
                            "operating_system": "precise",
                            "php": {
                                "version": "5.6",
                                "memory_limit": 128,
                                "apc": 96,
                                "max_execution_time": 300,
                                "max_post_size": 256,
                                "max_input_vars": 1000,
                                "sendmail_path": None,
                            },
                        },
                        "artifact": None,
                        "_links": {
                            "self": {
                                "href": "{baseUri}/environments/"
                                "24-a47ac10b-58cc-4372"
                                "-a567-0e02b2c3d470"
                            }
                        },
                    },
                    {
                        "id": "15-a47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                        "label": "Production",
                        "name": "prod",
                        "application": {
                            "name": "Sample SiteFactory application",
                            "uuid": "a47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                        },
                        "domains": [
                            "siteprod.hosted.acquia-sites.com",
                            "my-domain-prod.com",
                        ],
                        "gardener": {
                            "site-update": "my-domain-prod.com/admin/"
                            "gardens/site-update",
                            "multi-site": "my-domain-prod.com/"
                            "admin/gardens/staging",
                        },
                        "active_domain": "my-domain-prod.com",
                        "image_url": None,
                        "ips": ["10.0.1.1", "10.0.1.2"],
                        "region": "us-east-1",
                        "status": "normal",
                        "type": "drupal",
                        "size": None,
                        "weight": 0,
                        "vcs": {
                            "type": "git",
                            "path": "tags/01-01-2015",
                            "url": "site@svn-3.hosted."
                            "acquia-sites.com:site.git",
                        },
                        "insight": None,
                        "flags": {
                            "cde": False,
                            "hsd": False,
                            "livedev": False,
                            "multi_region": False,
                            "production": True,
                            "production_mode": True,
                            "remote_admin": False,
                            "varnish": True,
                            "varnish_over_ssl": False,
                            "service_management": False,
                        },
                        "configuration": None,
                        "artifact": None,
                        "_links": {
                            "self": {
                                "href": "{baseUri}/environments/"
                                "15-a47ac10b-58cc-4372"
                                "-a567-0e02b2c3d470"
                            }
                        },
                    },
                    {
                        "id": "32-a47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                        "label": "Stage",
                        "name": "test",
                        "application": {
                            "name": "Sample Node application",
                            "uuid": "a47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                        },
                        "domains": [
                            "sitetest.hosted.acquia-sites.com",
                            "my-domain-test.com",
                        ],
                        "active_domain": "my-domain-test.com",
                        "image_url": None,
                        "ips": [],
                        "region": "us-east-1",
                        "status": "normal",
                        "type": "node",
                        "size": "medium",
                        "weight": 20,
                        "vcs": {
                            "type": "git",
                            "path": None,
                            "url": "qa10@svn-3.networkdev."
                            "ahserversdev.com:qa10.git",
                        },
                        "insight": {
                            "status": "not-found",
                            "scores": None,
                            "counts": None,
                        },
                        "flags": {
                            "cde": False,
                            "hsd": False,
                            "livedev": False,
                            "multi_region": False,
                            "production": False,
                            "production_mode": False,
                            "remote_admin": False,
                            "varnish": True,
                            "varnish_over_ssl": False,
                            "service_management": False,
                        },
                        "configuration": {
                            "operating_system": "precise",
                            "node": {"version": "6.11.1"},
                        },
                        "artifact": {"id": 1, "name": "Example artifact"},
                        "_links": {
                            "self": {
                                "href": "{baseUri}/environments/"
                                "32-a47ac10b-58cc-4372"
                                "-a567-0e02b2c3d470"
                            }
                        },
                    },
                ]
            },
        }

        app_uuid = "a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/applications/{uuid}/environments".format(
            base_uri=self.endpoint, uuid=app_uuid
        )
        mocker.register_uri("GET", uri, json=response, status_code=200)

        envs = self.acquia.application(app_uuid).environments()
        self.assertIsInstance(envs, EnvironmentList)

    def test_application_permission(self, mocker):
        response = {
            "total": 53,
            "flags": {"administrator": False},
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/applications/"
                    "a47ac10b-58cc-4372-a567-0e02b2c3d470/permissions"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/applications/"
                    "a47ac10b-58cc-4372-a567-0e02b2c3d470"
                },
            },
            "_embedded": {
                "items": [
                    {
                        "name": "administer alerts",
                        "label": "Manage Insight alerts",
                        "description": "Grants the ability to ignore alerts "
                        "and change settings on a "
                        "Drupal website "
                        'using the "Fix it Now" feature.',
                        "group_label": "Insight",
                    },
                    {
                        "name": "revoke insight installs",
                        "title": "Revoke Insight sites",
                        "description": "Grants the ability to revoke sites so "
                        "they can no longer submit data.",
                        "group_label": "Insight",
                    },
                    {
                        "name": "deploy to non-prod",
                        "label": "Pull and deploy code, files, or databases "
                        "to non-production environments",
                        "description": "Grants the ability to enable or "
                        "disable live development "
                        "and non-production "
                        "work.",
                        "group_label": "Workflow",
                    },
                    {
                        "name": "deploy to prod",
                        "label": "Deploy code, files, or databases to "
                        "the production environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "pull from prod",
                        "label": "Pull files or databases from the "
                        "production environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "move file to non-prod",
                        "label": "Move files to non-production environments",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "move file to prod",
                        "label": "Move files to the production environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "move file from prod",
                        "label": "Move files from production environments",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "move file from non-prod",
                        "label": "Move files from non-production environments",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "clear varnish on non-prod",
                        "label": "Clear Varnish cache for "
                        "non-production environments",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "clear varnish on prod",
                        "label": "Clear Varnish cache for "
                        "the production environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "configure prod env",
                        "label": "Configure production environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "configure non-prod env",
                        "label": "Configure non-production environments",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "add an environment",
                        "label": "Add an environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "delete an environment",
                        "label": "Delete an environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "administer domain non-prod",
                        "label": "Add or remove domains for "
                        "non-production environments",
                        "description": None,
                        "group_label": "Domains",
                    },
                    {
                        "name": "administer domain prod",
                        "label": "Add or remove domains for the "
                        "production environment",
                        "description": None,
                        "group_label": "Domains",
                    },
                    {
                        "name": "administer ssl prod",
                        "label": "Add or remove SSL certificates for "
                        "the production environment",
                        "description": None,
                        "group_label": "Domains",
                    },
                    {
                        "name": "administer ssl non-prod",
                        "label": "Add or remove SSL certificates for "
                        "the non-production environments",
                        "description": None,
                        "group_label": "Domains",
                    },
                    {
                        "name": "reboot server",
                        "label": "Reboot server",
                        "description": None,
                        "group_label": "Server administration",
                    },
                    {
                        "name": "resize server",
                        "label": "Resize server",
                        "description": "Increasing the size of your "
                        "server costs money.",
                        "group_label": "Server administration",
                    },
                    {
                        "name": "suspend server",
                        "label": "Suspend server",
                        "description": None,
                        "group_label": "Server administration",
                    },
                    {
                        "name": "configure server",
                        "label": "Configure server",
                        "description": None,
                        "group_label": "Server administration",
                    },
                    {
                        "name": "download logs non-prod",
                        "label": "Download logs for non-production "
                        "environments",
                        "description": None,
                        "group_label": "Logs",
                    },
                    {
                        "name": "download logs prod",
                        "label": "Download logs for the production"
                        " environment",
                        "description": None,
                        "group_label": "Logs",
                    },
                    {
                        "name": "add database",
                        "label": "Add a database",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "remove database",
                        "label": "Remove a database",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "view database connection",
                        "label": "View database connection details"
                        " (username, password, or hostname)",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "download db backup non-prod",
                        "label": "Download database backups for "
                        "non-production environments",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "download db backup prod",
                        "label": "Download database backups for "
                        "the production environment",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "create db backup non-prod",
                        "label": "Create database backups for "
                        "non-production environments",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "create db backup prod",
                        "label": "Create database backups for the "
                        "production environment",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "restore db backup non-prod",
                        "label": "Restore database backups for "
                        "non-production environments",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "restore db backup prod",
                        "label": "Restore database backups for "
                        "the production environment",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "administer team",
                        "label": "Add or remove a user of a team",
                        "description": "Granting this permission "
                        "will give any user with this "
                        "role full permissions on this team.",
                        "group_label": "Administration",
                    },
                    {
                        "name": "access cloud api",
                        "label": "Access the Cloud API",
                        "description": "Grants the ability to use "
                        "the API and bypass all other "
                        "permissions via command line tools.",
                        "group_label": "Administration",
                    },
                    {
                        "name": "activate add on",
                        "label": "Activate an Acquia Subscription add-on",
                        "description": None,
                        "group_label": "Administration",
                    },
                    {
                        "name": "administer cron non-prod",
                        "label": "Modify cron tasks for "
                        "non-production environments",
                        "description": None,
                        "group_label": "Cron",
                    },
                    {
                        "name": "administer cron prod",
                        "label": "Modify cron tasks for the "
                        "production environment",
                        "description": None,
                        "group_label": "Cron",
                    },
                    {
                        "name": "search limit increase",
                        "label": "Increase the search index "
                        "limit for a subscription",
                        "description": None,
                        "group_label": "Search",
                    },
                    {
                        "name": "search schema edit",
                        "label": "Edit the search schema for a subscription",
                        "description": None,
                        "group_label": "Search",
                    },
                    {
                        "name": "create support ticket",
                        "label": "Create a support ticket",
                        "description": None,
                        "group_label": "Support",
                    },
                    {
                        "name": "edit any support ticket",
                        "label": "View and edit any support tickets "
                        "for a subscription",
                        "description": None,
                        "group_label": "Support",
                    },
                    {
                        "name": "administer ssh keys",
                        "label": "Manage SSH keys",
                        "description": "Grants the ability to use SSH "
                        "and bypass all other permissions "
                        "via command line tools.",
                        "group_label": "SSH keys",
                    },
                    {
                        "name": "view build plans",
                        "label": "View Build plans",
                        "description": None,
                        "group_label": "Build",
                    },
                    {
                        "name": "edit build plans",
                        "label": "Edit Build plans",
                        "description": None,
                        "group_label": "Build",
                    },
                    {
                        "name": "run build plans",
                        "label": "Run Build plans",
                        "description": None,
                        "group_label": "Build",
                    },
                    {
                        "name": "add ssh key to git",
                        "label": "Add SSH key to git repository",
                        "description": None,
                        "group_label": "SSH keys",
                    },
                    {
                        "name": "add ssh key to non-prod",
                        "label": "Add SSH key to non-production environments",
                        "description": None,
                        "group_label": "SSH keys",
                    },
                    {
                        "name": "add ssh key to prod",
                        "label": "Add SSH key to the production environment",
                        "description": None,
                        "group_label": "SSH keys",
                    },
                    {
                        "name": "view remote administration",
                        "label": "View Remote Administration",
                        "description": "This permission is only relevant if "
                        "your subscription has remote "
                        "administration.",
                        "group_label": "Administration",
                    },
                    {
                        "name": "edit remote administration",
                        "label": "Edit Remote Administration",
                        "description": "This permission is only relevant "
                        "if your subscription has remote "
                        "administration.",
                        "group_label": "Administration",
                    },
                ]
            },
        }

        app_uuid = "a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = f"{self.endpoint}/applications/{app_uuid}/permissions"
        mocker.register_uri("GET", uri, json=response, status_code=200)

        perms = self.acquia.application(app_uuid).permissions()

        self.assertEqual(perms.status_code, 200)
        self.assertIn(b"remote", perms.content)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Test Permissions Endpoint"""

import requests_mock

from acapi2.resources.permissionslist import PermissionsList
from acapi2.tests import BaseTest


@requests_mock.Mocker()
class TestPermissions(BaseTest):
    def test_permissions_list(self, mocker):
        response = {
            "total": 53,
            "_links": {
                "self": {"href": "https://cloud.acquia.com/api/permissions"}
            },
            "_embedded": {
                "items": [
                    {
                        "name": "administer alerts",
                        "label": "Manage Insight alerts",
                        "description": "Grants the ability to ignore alerts "
                        "and change settings on a "
                        "Drupal website "
                        "using the Fix it Now feature.",
                        "group_label": "Insight",
                    },
                    {
                        "name": "revoke insight installs",
                        "title": "Revoke Insight sites",
                        "description": "Grants the ability to revoke sites so "
                        "they can no longer submit data.",
                        "group_label": "Insight",
                    },
                    {
                        "name": "deploy to non-prod",
                        "label": "Pull and deploy code, files, or databases "
                        "to non-production environments",
                        "description": "Grants the ability to enable or "
                        "disable live development and "
                        "non-production "
                        "work.",
                        "group_label": "Workflow",
                    },
                    {
                        "name": "deploy to prod",
                        "label": "Deploy code, files, or databases to "
                        "the production environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "pull from prod",
                        "label": "Pull files or databases from the "
                        "production environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "move file to non-prod",
                        "label": "Move files to non-production environments",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "move file to prod",
                        "label": "Move files to the production environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "move file from prod",
                        "label": "Move files from production environments",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "move file from non-prod",
                        "label": "Move files from non-production environments",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "clear varnish on non-prod",
                        "label": "Clear Varnish cache for "
                        "non-production environments",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "clear varnish on prod",
                        "label": "Clear Varnish cache for the "
                        "production environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "configure prod env",
                        "label": "Configure production environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "configure non-prod env",
                        "label": "Configure non-production environments",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "add an environment",
                        "label": "Add an environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "delete an environment",
                        "label": "Delete an environment",
                        "description": None,
                        "group_label": "Workflow",
                    },
                    {
                        "name": "administer domain non-prod",
                        "label": "Add or remove domains for "
                        "non-production environments",
                        "description": None,
                        "group_label": "Domains",
                    },
                    {
                        "name": "administer domain prod",
                        "label": "Add or remove domains for the "
                        "production environment",
                        "description": None,
                        "group_label": "Domains",
                    },
                    {
                        "name": "administer ssl prod",
                        "label": "Add or remove SSL certificates "
                        "for the production environment",
                        "description": None,
                        "group_label": "Domains",
                    },
                    {
                        "name": "administer ssl non-prod",
                        "label": "Add or remove SSL certificates "
                        "for the non-production environments",
                        "description": None,
                        "group_label": "Domains",
                    },
                    {
                        "name": "reboot server",
                        "label": "Reboot server",
                        "description": None,
                        "group_label": "Server administration",
                    },
                    {
                        "name": "resize server",
                        "label": "Resize server",
                        "description": "Increasing the size of your "
                        "server costs money.",
                        "group_label": "Server administration",
                    },
                    {
                        "name": "suspend server",
                        "label": "Suspend server",
                        "description": None,
                        "group_label": "Server administration",
                    },
                    {
                        "name": "configure server",
                        "label": "Configure server",
                        "description": None,
                        "group_label": "Server administration",
                    },
                    {
                        "name": "download logs non-prod",
                        "label": "Download logs for non-production "
                        "environments",
                        "description": None,
                        "group_label": "Logs",
                    },
                    {
                        "name": "download logs prod",
                        "label": "Download logs for the production "
                        "environment",
                        "description": None,
                        "group_label": "Logs",
                    },
                    {
                        "name": "add database",
                        "label": "Add a database",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "remove database",
                        "label": "Remove a database",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "view database connection",
                        "label": "View database connection details "
                        "(username, password, or hostname)",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "download db backup non-prod",
                        "label": "Download database backups for "
                        "non-production environments",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "download db backup prod",
                        "label": "Download database backups for the "
                        "production environment",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "create db backup non-prod",
                        "label": "Create database backups for "
                        "non-production environments",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "create db backup prod",
                        "label": "Create database backups "
                        "for the production environment",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "restore db backup non-prod",
                        "label": "Restore database backups "
                        "for non-production environments",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "restore db backup prod",
                        "label": "Restore database backups "
                        "for the production environment",
                        "description": None,
                        "group_label": "Databases",
                    },
                    {
                        "name": "administer team",
                        "label": "Add or remove a user of a team",
                        "description": "Granting this permission will "
                        "give any user with this "
                        "role full permissions "
                        "on this team.",
                        "group_label": "Administration",
                    },
                    {
                        "name": "access cloud api",
                        "label": "Access the Cloud API",
                        "description": "Grants the ability to use the"
                        " API and bypass all other permissions "
                        "via command line tools.",
                        "group_label": "Administration",
                    },
                    {
                        "name": "activate add on",
                        "label": "Activate an Acquia Subscription add-on",
                        "description": None,
                        "group_label": "Administration",
                    },
                    {
                        "name": "administer cron non-prod",
                        "label": "Modify cron tasks for non-production "
                        "environments",
                        "description": None,
                        "group_label": "Cron",
                    },
                    {
                        "name": "administer cron prod",
                        "label": "Modify cron tasks for the production "
                        "environment",
                        "description": None,
                        "group_label": "Cron",
                    },
                    {
                        "name": "search limit increase",
                        "label": "Increase the search index limit for a "
                        "subscription",
                        "description": None,
                        "group_label": "Search",
                    },
                    {
                        "name": "search schema edit",
                        "label": "Edit the search schema for a subscription",
                        "description": None,
                        "group_label": "Search",
                    },
                    {
                        "name": "create support ticket",
                        "label": "Create a support ticket",
                        "description": None,
                        "group_label": "Support",
                    },
                    {
                        "name": "edit any support ticket",
                        "label": "View and edit any support tickets "
                        "for a subscription",
                        "description": None,
                        "group_label": "Support",
                    },
                    {
                        "name": "administer ssh keys",
                        "label": "Manage SSH keys",
                        "description": "Grants the ability to use SSH and "
                        "bypass all other permissions via "
                        "command line tools.",
                        "group_label": "SSH keys",
                    },
                    {
                        "name": "view build plans",
                        "label": "View Build plans",
                        "description": None,
                        "group_label": "Build",
                    },
                    {
                        "name": "edit build plans",
                        "label": "Edit Build plans",
                        "description": None,
                        "group_label": "Build",
                    },
                    {
                        "name": "run build plans",
                        "label": "Run Build plans",
                        "description": None,
                        "group_label": "Build",
                    },
                    {
                        "name": "add ssh key to git",
                        "label": "Add SSH key to git repository",
                        "description": None,
                        "group_label": "SSH keys",
                    },
                    {
                        "name": "add ssh key to non-prod",
                        "label": "Add SSH key to non-production environments",
                        "description": None,
                        "group_label": "SSH keys",
                    },
                    {
                        "name": "add ssh key to prod",
                        "label": "Add SSH key to the production environment",
                        "description": None,
                        "group_label": "SSH keys",
                    },
                    {
                        "name": "view remote administration",
                        "label": "View Remote Administration",
                        "description": "This permission is only relevant if "
                        "your subscription has remote "
                        "administration.",
                        "group_label": "Administration",
                    },
                    {
                        "name": "edit remote administration",
                        "label": "Edit Remote Administration",
                        "description": "This permission is only relevant if "
                        "your subscription has remote "
                        "administration.",
                        "group_label": "Administration",
                    },
                ]
            },
        }

        uri = f"{self.endpoint}/permissions"
        mocker.register_uri("GET", uri, json=response, status_code=200)
        permissions = self.acquia.permissions()
        self.assertIsInstance(permissions, PermissionsList)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Acquia Cloud API notifications."""

from acapi2.exceptions import AcquiaCloudNoDataException
from acapi2.resources.acquialist import AcquiaList
from acapi2.resources.notification import Notification


class NotificationList(AcquiaList):
    def __init__(
        self,
        uri: str,
        api_key: str,
        api_secret: str,
        qry_params: dict = None,
        *args,
        **kwargs,
    ) -> None:

        super().__init__(uri, api_key, api_secret, *args, **kwargs)
        self._qry_params = qry_params
        self.fetch()

    def fetch(self) -> None:
        notifications = self.request(
            uri=self.uri, params=self._qry_params
        ).json()
        try:
            notif_items = notifications["_embedded"]["items"]
        except KeyError:
            raise AcquiaCloudNoDataException()
        else:
            for notification in notif_items:
                notif_uri = notification["_links"]["self"]["href"]
                self.__setitem__(
                    notification["uuid"],
                    Notification(notif_uri, self.api_key, self.api_secret),
                )

    @property
    def base_uri(self) -> str:
        return self._base_uri

    @base_uri.setter
    def base_uri(self, base_uri: str):
        self._base_uri = f"{base_uri}/notifications"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Subscriptions tests"""

import requests_mock

from acapi2.resources.applicationlist import ApplicationList
from acapi2.resources.subscriptionlist import SubscriptionList
from acapi2.tests import BaseTest


@requests_mock.Mocker()
class TestSubscriptions(BaseTest):
    def test_subscription_list(self, mocker):
        response = {
            "total": 3,
            "_links": {
                "self": {"href": "{baseUri}/subscriptions"},
                "sort": {
                    "href": "{baseUri}/subscriptions{?sort}",
                    "templated": True,
                },
                "filter": {
                    "href": "{baseUri}/subscriptions{?filter}",
                    "templated": True,
                },
                "limit": {
                    "href": "{baseUri}/subscriptions{?limit}",
                    "templated": True,
                },
                "parent": {"href": "{baseUri}"},
            },
            "_embedded": {
                "items": [
                    {
                        "id": 123,
                        "uuid": "a1aaa1aa-aa11-aaa1" "-11aa-123456789012",
                        "name": "Acquia Cloud Free Subscription",
                        "start_at": "2011-03-28T00:00:00",
                        "expire_at": "2015-11-11T00:00:00",
                        "product": {
                            "id": 1890149,
                            "name": "Acquia Cloud Free",
                            "type": "free",
                        },
                        "applications_total": 3,
                        "applications_used": 1,
                        "advisory_hours_total": 1.23,
                        "advisory_hours_used": 4.56,
                        "organization": {
                            "uuid": "g1ggg1gg-" "gg11-ggg1-11gg-123456789012",
                            "name": "Acquia Inc.",
                        },
                        "flags": {
                            "active": True,
                            "expired": False,
                            "zuora": False,
                        },
                        "_links": {
                            "self": {
                                "href": "{baseUri}/subscriptions/"
                                "a1aaa1aa-aa11"
                                "-aaa1-11aa-123456789012"
                            }
                        },
                    },
                    {
                        "id": 222,
                        "uuid": "b1bbb1bb-bb11-bbb1-" "11bb-123456789012",
                        "name": "My Acquia Subscription",
                        "start_at": "2012-05-15T12:00:00Z",
                        "expire_at": "2015-05-15T12:00:00Z",
                        "product": {
                            "id": 8999,
                            "name": "Enterprise",
                            "type": "enterprise",
                        },
                        "applications_total": 5,
                        "applications_used": 2,
                        "advisory_hours_total": 2.34,
                        "advisory_hours_used": 5.67,
                        "organization": {
                            "uuid": "g2ggg2gg-gg22-ggg2-" "22gg-123456789012",
                            "name": "My Organization",
                        },
                        "flags": {
                            "active": True,
                            "expired": False,
                            "zuora": False,
                        },
                        "_links": {
                            "self": {
                                "href": "{baseUri}/subscriptions/"
                                "b1bbb1bb-bb11"
                                "-bbb1-11bb-123456789012"
                            }
                        },
                    },
                    {
                        "id": 333,
                        "uuid": "c1ccc1cc-cc11-ccc1" "-11cc-123456789012",
                        "name": "My Acquia Subscription 2",
                        "start_at": "2012-05-15T12:00:00Z",
                        "expire_at": "2015-05-15T12:00:00Z",
                        "product": {
                            "id": 8999,
                            "name": "Enterprise",
                            "type": "enterprise",
                        },
                        "applications_total": 5,
                        "applications_used": 2,
                        "advisory_hours_total": 3.45,
                        "advisory_hours_used": 6.78,
                        "organization": {
                            "uuid": "g2ggg2gg-gg22-" "ggg2-22gg-123456789012",
                            "name": "My Organization",
                        },
                        "flags": {
                            "active": True,
                            "expired": False,
                            "zuora": False,
                        },
                        "_links": {
                            "self": {
                                "href": "{baseUri}/subscriptions/"
                                "c1ccc1cc-cc11-"
                                "ccc1-11cc-123456789012"
                            }
                        },
                    },
                ]
            },
        }

        uri = "{}/subscriptions".format(self.endpoint)
        mocker.register_uri("GET", uri, json=response, status_code=200)

        subscriptions = self.acquia.subscriptions()
        self.assertIsInstance(subscriptions, SubscriptionList)

    def test_subscription_applications(self, mocker):
        response = {
            "total": 2,
            "pagination": {"total": 2, "limit": 10, "offset": 0},
            "_links": {
                "self": {
                    "href": "{baseUri}/subscriptions/abcdef12-1234"
                    "-4372-a567-0e02b2c3d470/applications"
                },
                "parent": {
                    "href": "{baseUri}/subscriptions/abcdef12-1234"
                    "-4372-a567-0e02b2c3d470"
                },
                "sort": {
                    "href": "{baseUri}/subscriptions/abcdef12-1234"
                    "-4372-a567-0e02b2c3d470/"
                    "applications{?sort}",
                    "templated": True,
                },
                "filter": {
                    "href": "{baseUri}/subscriptions/abcdef12-1234"
                    "-4372-a567-0e02b2c3d470/"
                    "applications{?filter}",
                    "templated": True,
                },
                "limit": {
                    "href": "{baseUri}/subscriptions/abcdef12-1234"
                    "-4372-a567-0e02b2c3d470/"
                    "applications{?limit}",
                    "templated": True,
                },
            },
            "_embedded": {
                "items": [
                    {
                        "id": 241643,
                        "uuid": "a47ac10b-58cc-4372-" "a567-0e02b2c3d470",
                        "name": "Sample application 1",
                        "hosting": {"type": "acp", "id": "devcloud:devcloud2"},
                        "subscription": {
                            "uuid": "s47ac10b-58cc-4372-" "a567-0e02b2c3d470",
                            "name": "Sample subscription",
                        },
                        "organization": {
                            "uuid": "g47ac10b-58cc-4372-" "a567-0e02b2c3d470",
                            "name": "Sample organization",
                        },
                        "flags": {"remote_admin": True},
                        "status": "normal",
                        "_links": {
                            "self": {
                                "href": "{baseUri}/applications/"
                                "a47ac10b-58cc-4372"
                                "-a567-0e02b2c3d470"
                            }
                        },
                    },
                    {
                        "id": 954291,
                        "uuid": "a47ac10b-58cc-4372" "-a567-0e02b2c3d471",
                        "name": "Sample application 2",
                        "hosting": {
                            "type": "free",
                            "id": "devcloud:devcloud2",
                        },
                        "subscription": {
                            "uuid": "s47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                            "name": "Sample subscription",
                        },
                        "organization": {
                            "uuid": "g47ac10b-58cc-4372" "-a567-0e02b2c3d470",
                            "name": "Sample organization",
                        },
                        "flags": {"remote_admin": False},
                        "status": "provisioning",
                        "_links": {
                            "self": {
                                "href": "{baseUri}/applications/"
                                "a47ac10b-58cc-4372"
                                "-a567-0e02b2c3d471"
                            }
                        },
                    },
                ]
            },
        }

        uuid = "a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/subscriptions/{uuid}/applications".format(
            base_uri=self.endpoint, uuid=uuid
        )
        mocker.register_uri("GET", uri, json=response, status_code=200)

        applications = self.acquia.subscription(uuid).applications()
        self.assertIsInstance(applications, ApplicationList)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Agreements tests"""

import requests_mock

from acapi2.resources.agreement import Agreement
from acapi2.resources.agreementlist import AgreementList
from acapi2.tests import BaseTest


@requests_mock.Mocker()
class TestAgreements(BaseTest):
    def test_agreement_list(self, mocker):
        response = {
            "total": 3,
            "_links": {
                "self": {"href": "https://cloud.acquia.com/api/agreements"},
                "parent": {"href": "https://cloud.acquia.com/api/"},
            },
            "_embedded": {
                "items": [
                    {
                        "uuid": "efc62c93-8203-4e8b-a8ff-4d18b780d4ab",
                        "document_uuid": "f25d0284-f25f-4e59-"
                        "9c48-7c39ae57b400",
                        "title": "Agreement Title",
                        "body": "<p>Agreement body and text.</p>",
                        "status": "accepted",
                        "created_at": "2017-01-23T12:00:00Z",
                        "updated_at": "2017-01-27T12:00:00Z",
                        "actioned_by": {
                            "uuid": "5aa902c5-f1c1-6c94-edfa-86bc58d0dce3",
                            "first_name": "James",
                            "last_name": "Kirk",
                            "mail": "james.kirk@example.com",
                            "picture_url": "https://accounts.acquia.com/sites"
                            "/default/avatars/456def"
                            "?mail=james.kirk"
                            "@example.com",
                            "username": "james.kirk",
                        },
                        "reference": {
                            "uuid": "9ab09eba-290d-4ed9-be4d-fa194ab92f39",
                            "name": "Acquia Subscription",
                            "type": "subscription",
                        },
                        "_links": {
                            "self": {
                                "href": "https://cloud.acquia.com/api"
                                "/agreements/efc62c93-8203-4e8b-"
                                "a8ff-4d18b780d4ab"
                            }
                        },
                    },
                    {
                        "uuid": "b63fff64-6c18-4899-acba-00ec6c8930e9",
                        "title": "Another Agreement",
                        "body": "<p>This is the body and text of another "
                        "agreement.</p>",
                        "status": "declined",
                        "created_at": "2017-02-23T12:00:00Z",
                        "updated_at": "2017-02-27T12:00:00Z",
                        "actioned_by": {
                            "uuid": "550e8400-e29b-41d4-a716-446655440000",
                            "first_name": "Jane",
                            "last_name": "Doe",
                            "mail": "jane.doe@example.com",
                            "picture_url": "https://accounts.acquia.com/"
                            "sites/default/avatars/123abc?"
                            "mail=jane.doe"
                            "@example.com",
                            "username": "jane.doe",
                        },
                        "reference": {
                            "uuid": "9ab09eba-290d-4ed9-be4d-fa194ab92f39",
                            "name": "Acquia Subscription",
                            "type": "subscription",
                        },
                        "_links": {
                            "self": {
                                "href": "https://cloud.acquia.com/api/"
                                "agreements/b63fff64-6c18-4899-acba-"
                                "00ec6c8930e9"
                            }
                        },
                    },
                    {
                        "uuid": "a8777880-8924-494a-abe2-62cc092df269",
                        "title": "A Third Agreement",
                        "body": "<p>This is the body and text of one "
                        "more agreement.</p>",
                        "status": "pending",
                        "created_at": "2017-02-23T12:00:00Z",
                        "updated_at": None,
                        "actioned_by": None,
                        "reference": {
                            "uuid": "9ab09eba-290d-4ed9-be4d-fa194ab92f39",
                            "name": "Acquia Subscription",
                            "type": "subscription",
                        },
                        "_links": {
                            "self": {
                                "href": "https://cloud.acquia.com/api/"
                                "agreements/a8777880-8924-494a-"
                                "abe2-62cc092df269"
                            }
                        },
                    },
                ]
            },
        }

        uri = f"{self.endpoint}/agreements"
        mocker.register_uri("GET", uri, json=response, status_code=200)

        agrs = self.acquia.agreements()
        self.assertIsInstance(agrs, AgreementList)

    def test_agreement(self, mocker):
        uuid = "efc62c93-8203-4e8b-a8ff-4d18b780d4ab"
        response = {
            "uuid": uuid,
            "document_uuid": "f25d0284-f25f-4e59-9c48-7c39ae57b400",
            "title": "Agreement Title",
            "body": "<p>Agreement body and text.</p>",
            "status": "accepted",
            "created_at": "2017-01-23T12:00:00Z",
            "updated_at": "2017-01-27T12:00:00Z",
            "actioned_by": {
                "uuid": "5aa902c5-f1c1-6c94-edfa-86bc58d0dce3",
                "first_name": "James",
                "last_name": "Kirk",
                "mail": "james.kirk@example.com",
                "picture_url": "https://accounts.acquia.com/sites/default/"
                "avatars/456def?mail=james.kirk@example.com",
                "username": "james.kirk",
            },
            "reference": {
                "uuid": "9ab09eba-290d-4ed9-be4d-fa194ab92f39",
                "name": "Acquia Subscription",
                "type": "subscription",
            },
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/agreements/"
                    "efc62c93-8203-4e8b-a8ff-4d18b780d4ab"
                },
                "invitees": {
                    "href": "https://cloud.acquia.com/api/agreements/"
                    "efc62c93-8203-4e8b-a8ff-4d18b780d4ab/invitees"
                },
                "subscription": {
                    "href": "https://cloud.acquia.com/api/subscriptions/"
                    "9ab09eba-290d-4ed9-be4d-fa194ab92f39"
                },
                "actioned_by": {
                    "href": "https://cloud.acquia.com/api/users/"
                    "5aa902c5-f1c1-6c94-edfa-86bc58d0dce3"
                },
                "parent": {"href": "https://cloud.acquia.com/api/agreements"},
            },
        }

        uri = f"{self.endpoint}/agreements/{uuid}"
        mocker.register_uri("GET", uri, json=response, status_code=200)
        agreement = self.acquia.agreement(uuid)
        self.assertIsInstance(agreement, Agreement)

    def test_accept_agreement(self, mocker):
        response = {"message": "The agreement has been accepted."}
        uuid = "efc62c93-8203-4e8b-a8ff-4d18b780d4ab"
        uri = f"{self.endpoint}/agreements/{uuid}/actions/accept"
        mocker.register_uri(
            url=uri, method="POST", status_code=200, json=response
        )

        response = self.acquia.agreement(uuid).accept()
        self.assertEqual(response.status_code, 200)

    def test_decline_agreement(self, mocker):
        response = {"message": "The agreement has been declined."}
        uuid = "efc62c93-8203-4e8b-a8ff-4d18b780d4ab"
        uri = f"{self.endpoint}/agreements/{uuid}/actions/decline"
        mocker.register_uri(
            url=uri, method="POST", status_code=200, json=response
        )

        response = self.acquia.agreement(uuid).decline()
        self.assertEqual(response.status_code, 200)

    def test_agreement_invitees(self, mocker):
        response = {
            "total": 2,
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/agreements/"
                    "efc62c93-8203-4e8b-a8ff-4d18b780d4ab/invitees"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/agreements/"
                    "efc62c93-8203-4e8b-a8ff-4d18b780d4ab"
                },
            },
            "_embedded": {
                "items": [
                    {
                        "uuid": "u4ee550f-ee0c-102e-8305-1231390f2cc1",
                        "first_name": "User",
                        "last_name": "One",
                        "mail": "user1@example.com",
                        "username": "user.one",
                        "picture_url": "https://accounts.acquia.com/"
                        "path/to/image.png",
                    },
                    {
                        "uuid": "u4ef8edc-ee0c-102e-8305-1231390f2cc2",
                        "first_name": "User",
                        "last_name": "Two",
                        "mail": "user2@example.com",
                        "username": "user.two",
                        "picture_url": "https://accounts.acquia.com/"
                        "path/to/image.png",
                    },
                ]
            },
        }
        uuid = "efc62c93-8203-4e8b-a8ff-4d18b780d4ab"

        uri = f"{self.endpoint}/agreements/{uuid}/invitees"
        mocker.register_uri("GET", uri, json=response, status_code=200)

        invitees_response = self.acquia.agreement(uuid).invitees()

        self.assertIn("total", invitees_response)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from unittest.mock import patch

from acapi2.http_request import HttpRequest
from acapi2.tests import BaseTest


class TestHttpRequest(BaseTest):
    def test_session(self):
        http_request = HttpRequest()
        http_request_1 = HttpRequest()
        self.assertEqual(id(http_request.session), id(http_request_1.session))

    def test_get_session(self):
        request_session = HttpRequest()._get_session()
        self.assertEqual(HttpRequest._session, request_session)

    @patch("requests.Session.request")
    def test_make_request(self, mock_session):
        http_request = HttpRequest()
        http_request.body = "body"
        http_request.do()
        mock_session.assert_called_once_with(
            "GET", "http://localhost/", data="body", headers={}
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Notification object"""


import logging
import time
from datetime import datetime, timedelta

import requests_cache

from acapi2.exceptions import (
    AcquiaCloudNotificationFailedException,
    AcquiaCloudTimeoutError,
)
from acapi2.resources.acquiaresource import AcquiaResource

LOGGER = logging.getLogger("acapi2.resources.notification")


class Notification(AcquiaResource):
    POLL_INTERVAL = 3

    def __init__(
        self,
        uri: str,
        api_key: str,
        api_secret: str,
        filters: dict = None,
        *args,
        **kwargs,
    ) -> None:
        super().__init__(uri, api_key, api_secret, *args, **kwargs)

    def pending(self) -> bool:
        """Check if a notification is still pending.

        :return: is the notification task in progress or no
        """
        with requests_cache.disabled():
            notification = self.request().json()
        self.data = notification
        return notification["status"] == "in-progress"

    def wait(self, timeout: int = 1800) -> "Notification":
        """Wait for a notification task to finish executing.

        :param timeout: the max number of seconds to wait for the notification
                        to complete
        :return: Notification object or raise exceptions
        """
        start = datetime.now()
        max_time = start + timedelta(seconds=timeout)
        while self.pending():
            # Ensure the timeout hasn't been exceeded.
            if datetime.now() >= max_time:
                msg = (
                    f"Time out of exceeded while waiting "
                    f"for {self.data['uuid']}"
                )
                raise AcquiaCloudTimeoutError(msg, self.data)
            time.sleep(self.POLL_INTERVAL)

        # Grab the cached response
        notification = self.data
        if notification["status"] in ["failed", "error"]:
            raise AcquiaCloudNotificationFailedException(
                f"Notification {notification['uuid']} failed", notification
            )

        end = datetime.now()
        delta = end - start
        LOGGER.info(
            "Waited %.2f seconds for notification to complete",
            delta.total_seconds(),
        )

        return self



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Acquia API task queue resource."""
import logging
import time
from datetime import datetime, timedelta

import requests_cache

from acapi2.exceptions import AcquiaCloudTaskFailedException
from acapi2.resources.acquiaresource import AcquiaResource

LOGGER = logging.getLogger("acapi2.resources.task")


class Task(AcquiaResource):
    """
    Task is deprecated, let's not use it anymore.
    """

    POLL_INTERVAL = 3

    def mangle_uri(self, uri: str, task_data: dict):
        raise NotImplementedError

    def is_pending(self) -> bool:
        with requests_cache.disabled():
            tasks = self.request().json()["embedded"]["_items"]

        self.data = tasks[self.__getitem__("uuid")]
        return self.data["status"] != "in-progress"

    def wait(self, timeout: int = 1000) -> "Task":
        start = datetime.now()
        max_time = start + timedelta(seconds=timeout)

        while self.is_pending():
            if start >= max_time:
                msg = (
                    f"Time out exceeded while "
                    f"waiting for {self.data['uuid']}"
                )
                raise AcquiaCloudTaskFailedException(msg, self.data)
            time.sleep(self.POLL_INTERVAL)

        task = self.data
        if task["status"] == "failed":
            msg = f"Task {self.data['uuid']} failed"
            raise AcquiaCloudTaskFailedException(msg, task)

        end = datetime.now()
        delta = end - start

        LOGGER.info(
            "Waited %.2f seconds for task to complete", delta.total_seconds()
        )

        return self



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Agreement resource"""


from requests.exceptions import RequestException
from requests.sessions import Session

from acapi2.resources.acquiaresource import AcquiaResource


class Agreement(AcquiaResource):
    def __init__(
        self, uri: str, api_key: str, api_secret: str, *args, **kwargs
    ) -> None:
        super().__init__(uri, api_key, api_secret, *args, **kwargs)

    def accept(self) -> Session:
        """Accept a legal agreement by UUID."""
        uri = f"{self.uri}/actions/accept"
        response = Session()
        try:
            response = self.request(uri=uri, method="POST", data={})
        except RequestException:
            print("There was an error in the request.")

        return response

    def decline(self) -> Session:
        """Decline a legal agreement by UUID."""
        uri = f"{self.uri}/actions/decline"
        response = Session()
        try:
            response = self.request(uri=uri, method="POST", data={})
        except RequestException:
            print("There was an error in the request.")

        return response

    def invitees(self) -> Session:
        """Return a list of users invited to action this agreement."""
        uri = f"{self.uri}/invitees"
        return self.request(uri=uri).json()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Acquia Environment resource"""

from acapi2.exceptions import AcquiaCloudNoDataException
from acapi2.resources.acquialist import AcquiaList


class EnvironmentList(AcquiaList):
    def __init__(
        self,
        uri: str,
        api_key: str,
        api_secret: str,
        qry_params: dict = None,
        *args,
        **kwargs,
    ) -> None:
        super().__init__(uri, api_key, api_secret, *args, **kwargs)
        self._qry_params = qry_params
        self.fetch()

    def fetch(self):
        envs = self.request(uri=self.uri, params=self._qry_params).json()
        try:
            env_items = envs["_embedded"]["items"]
        except KeyError:
            raise AcquiaCloudNoDataException
        else:
            for env in env_items:
                name = env["name"]
                self.__setitem__(name, env)

    @property
    def base_uri(self) -> str:
        return self._base_uri

    @base_uri.setter
    def base_uri(self, base_uri: str):
        self._base_uri = f"{base_uri}/environments"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Acquia Cloud API resource base."""

from acapi2.resources.acquiadata import AcquiaData


class AcquiaResource(AcquiaData):
    def __getitem__(self, key):
        if not key and not self.data:
            self.populate_data()

        return self.data[key]

    @property
    def data(self):
        if not self._data:
            self.populate_data()
        return self._data

    @data.setter
    def data(self, data: dict):
        self._data = data

    def populate_data(self) -> None:
        results = self.request().json()
        self.data = results
